SyncView

Struct SyncView 

pub struct SyncView<T>
where T: ?Sized,
{ /* private fields */ }
🔬This is a nightly-only experimental API. (exclusive_wrapper)
SyncView provides mutable access, also referred to as exclusive access to the underlying value. However, it only permits immutable, or shared access to the underlying value when that value is Sync.

While this may seem not very useful, it allows SyncView to unconditionally implement Sync. Indeed, the safety requirements of Sync state that for SyncView to be Sync, it must be sound to share across threads, that is, it must be sound for &SyncView to cross thread boundaries. By design, a &SyncView<T> for non-Sync T has no API whatsoever, making it useless, thus harmless, thus memory safe.

Certain constructs like Futures can only be used with exclusive access, and are often Send but not Sync, so SyncView can be used as hint to the Rust compiler that something is Sync in practice.

Examples

Using a non-Sync future prevents the wrapping struct from being Sync:

use core::cell::Cell;

async fn other() {}
fn assert_sync<T: Sync>(t: T) {}
struct State<F> {
    future: F
}

assert_sync(State {
    future: async {
        let cell = Cell::new(1);
        let cell_ref = &cell;
        other().await;
        let value = cell_ref.get();
    }
});

SyncView ensures the struct is Sync without stripping the future of its functionality:

#![feature(exclusive_wrapper)]
use core::cell::Cell;
use core::sync::SyncView;

async fn other() {}
fn assert_sync<T: Sync>(t: T) {}
struct State<F> {
    future: SyncView<F>
}

assert_sync(State {
    future: SyncView::new(async {
        let cell = Cell::new(1);
        let cell_ref = &cell;
        other().await;
        let value = cell_ref.get();
    })
});

Parallels with a mutex

In some sense, SyncView can be thought of as a compile-time version of a mutex, as the borrow-checker guarantees that only one &mut can exist for any value. This is a parallel with the fact that & and &mut references together can be thought of as a compile-time version of a read-write lock.

Implementations

Source

impl<T> SyncView<T>

pub const fn new(t: T) -> SyncView<T>

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Wrap a value in an SyncView

pub const fn into_inner(self) -> T

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Unwrap the value contained in the SyncView

Source

impl<T> SyncView<T>
where T: ?Sized,

pub const fn as_pin_mut(self: Pin<&mut SyncView<T>>) -> Pin<&mut T>

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Gets pinned exclusive access to the underlying value.

SyncView is considered to structurally pin the underlying value, which means unpinned SyncViews can produce unpinned access to the underlying value, but pinned SyncViews only produce pinned access to the underlying value.

pub const fn from_mut(r: &mut T) -> &mut SyncView<T>

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Build a mutable reference to an SyncView<T> from a mutable reference to a T. This allows you to skip building an SyncView with SyncView::new.

pub const fn from_pin_mut(r: Pin<&mut T>) -> Pin<&mut SyncView<T>>

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Build a pinned mutable reference to an SyncView<T> from a pinned mutable reference to a T. This allows you to skip building an SyncView with SyncView::new.

Source

impl<T> SyncView<T>
where T: Sync + ?Sized,

pub const fn as_pin_ref(self: Pin<&SyncView<T>>) -> Pin<&T>

🔬This is a nightly-only experimental API. (exclusive_wrapper)

Gets pinned shared access to the underlying value.

SyncView is considered to structurally pin the underlying value, which means unpinned SyncViews can produce unpinned access to the underlying value, but pinned SyncViews only produce pinned access to the underlying value.