ListableSubstateDatabase

Trait ListableSubstateDatabase 

pub trait ListableSubstateDatabase {
    // Required method
    fn list_partition_keys(
        &self,
    ) -> Box<dyn Iterator<Item = DbPartitionKey> + '_>;
}
A partition listing interface between Track and a database vendor.

Required Methods

fn list_partition_keys(&self) -> Box<dyn Iterator<Item = DbPartitionKey> + '_>

Iterates over all partition keys, in an arbitrary order.

Alternatives

You likely want to use the read_partition_keys method instead, which returns an unmapped key. This is available if the trait ListableSubstateDatabaseExtensions is in scope.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl<D> ListableSubstateDatabase for StateTreeUpdatingDatabase<D>

Source

impl<S, D> ListableSubstateDatabase for SubstateDatabaseOverlay<S, D>

Implementors